#! /usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;

use LWP::UserAgent;
use HTTP::Cookies;
use Term::ReadKey;
use File::Basename qw(basename dirname);
use File::Copy;
use Getopt::Long;
use POSIX qw(strftime);
use Data::Dumper;

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] <mediawiki_file>[, <mediawiki_file>]*
Options can be:
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);

# Get user authentification information
print "Enter login:\n";
my $user = <STDIN>;
chomp $user;
print "Enter password:\n";
ReadMode 'noecho';
my $passwd = ReadLine 0;
chomp $passwd;
ReadMode 'normal';

# Create a user agent object
my $ua = LWP::UserAgent->new(env_proxy => 1);
$ua->agent("$0 (gibus perl script to remotely insert a page into http://www.laquadrature.net/wiki/)");
$ua->cookie_jar({});
  
# Authentification stored in cookie
my $auth = HTTP::Request->new(POST => "http://www.laquadrature.net/wiki/index?title=Special:Userlogin&action=submitlogin&type=login&returnto=Accueil");
$auth->content_type('application/x-www-form-urlencoded');
$auth->content("wpName=$user&wpPassword=$passwd");
my $res_auth = $ua->request($auth);
unless (($res_auth->is_success or $res_auth->is_redirect) and $res_auth->content eq '') {
  # Hack Capcha
  if ($res_auth->content =~ /<label for="wpCaptchaWord">([^<]+)<\/label>/) {
    die "Wrong user login/password capcha=!$1!\n!".$res_auth->content."!.\n";
  }
  die "Wrong user login/password !".$res_auth->content."!.\n";
}
print "OK identified\n";

foreach my $file (@ARGV) {
  # Define page to insert
  my $title = basename($file);
  $title =~ s/\.mediawiki$/\/en/o;
  my $body;
  # Slurp file
  {
    local $/ = undef;
    open FILE, $file or die "Cannot open $file: $!\n";
    $body = <FILE>;
    close FILE;
  }

  # First request to display form in order to fetch hidden parameters.
  my $url = "http://www.laquadrature.net/wiki/index?title=$title&action=edit";
  my $req = HTTP::Request->new(POST => $url);
  $req->content_type('application/x-www-form-urlencoded');
  my $res_first = $ua->request($req);
  unless ($res_first->is_success or $res_first->is_redirect) {
    die "Error in first request !".$res_first->message."! code=!".$res_first->code."!\n";
  }
  (my $wpEditToken) = $res_first->content =~ /type='hidden' value="([^"]*)" name="wpEditToken"/o;
  (my $wpEdittime) = $res_first->content =~ /type="hidden" value="([^"]*)" name="wpEdittime"/o;
  $wpEdittime = strftime("%Y%m%d%H%M%S",localtime(time-60)) unless $wpEdittime;
  #$wpEdittime = strftime("%Y%m%d%H%M%S",localtime) unless $wpEdittime;
  #$wpEdittime = '20080531162522' unless $wpEdittime;

  # Second request to insert the new page
  $url =~ s/action=edit/action=submit/;
  my $res = $ua->post($url, [
                              wpEdittime => $wpEdittime,
                              wpTextbox1 => $body,
                              wpEditToken => $wpEditToken,
                              wpWatchthis => 'true',
                              wpRecreate => 'true',
                            ]);
  unless (($res->is_success or $res->is_redirect) and $res->content eq '') {
    die "Error in second request !".$res->message."! code=!".$res->code."! page=!".$res->content."!\n";
  }

  # Third request to protect page from editing by anonymous users
  $url =~ s/action=submit/action=protect/;
  my $res_protect = $ua->post($url, [
                                      "mwProtect-level-edit" => "autoconfirmed",
                                      "mwProtect-level-move" => "autoconfirmed",
                                      wpEditToken => $wpEditToken,
                                      mwProtectWatch => 'true',
                                    ]);
  unless (($res_protect->is_success or $res_protect->is_redirect) and $res_protect->content eq '') {
    die "Error in third request !".$res_protect->message."! code=!".$res_protect->code."! page=!".$res_protect->content."!\n";
  }

  # Move page to uploaded directory
  (my $uploaded_dir = dirname($file)) =~ s/mediawiki/uploaded/;
  mkdir $uploaded_dir unless -d $uploaded_dir;
  move $file, $uploaded_dir;
  print "OK page $title inserted & moved to $uploaded_dir.\n";
}
