#! /usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;

use LWP::UserAgent;
use HTTP::Cookies;
use Term::ReadKey;
use File::Basename qw(basename dirname);
use Cwd 'abs_path';
use File::Copy;
use Getopt::Long;
use Data::Dumper;

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
    'help|h'         =>
        sub {
            print STDERR <<USAGE;
Usage: $0 [-h] <image_file>[, <image_file>]*
Options can be:
  --help, -h:                        print this message
USAGE
            exit 0;
        }
);

# Get user authentification information
print "Enter login:\n";
my $user = <STDIN>;
chomp $user;
print "Enter password:\n";
ReadMode 'noecho';
my $passwd = ReadLine 0;
chomp $passwd;
ReadMode 'normal';

# Create a user agent object
my $ua = LWP::UserAgent->new(env_proxy => 1);
$ua->agent("$0 (gibus perl script to remotely upload an image into http://www.laquadrature.net/wiki/)");
$ua->cookie_jar({});

# Authentification stored in cookie
my $auth = HTTP::Request->new(POST => "http://www.laquadrature.net/wiki/index.php?title=Special:Userlogin&action=submitlogin&type=login&returnto=Accueil");
$auth->content_type('application/x-www-form-urlencoded');
$auth->content("wpName=$user&wpPassword=$passwd");
my $res_auth = $ua->request($auth);
unless (($res_auth->is_success or $res_auth->is_redirect) and $res_auth->content eq '') {
    # Hack Captcha
    if ($res_auth->content =~ /<label for="wpCaptchaWord">([^<]+)<\/label>/) {
        my $captcha = $1;
        my $res_captcha = eval $captcha;
        (my $captcha_id) = ($res_auth->content =~ /id="wpCaptchaId" value="([^"]+)"/);
        warn "Wrong user login/password retrying with captcha=!$captcha! !$res_captcha! captchaid=!$captcha_id!.\n";
        my $auth_captcha = HTTP::Request->new(POST => "http://www.laquadrature.net/wiki/index.php?title=Special:Userlogin&action=submitlogin&type=login&returnto=Accueil");
        $auth_captcha->content("wpName=$user&wpPassword=$passwd&wpCaptchaWord=$res_captcha&wpCaptchaId=$captcha_id");
        my $res_auth_captcha = $ua->request($auth_captcha);
        unless (($res_auth_captcha->is_success or $res_auth_captcha->is_redirect) and $res_auth_captcha->content eq '') {
            die "Wrong user login/password !".$res_auth_captcha->content."!.\n";
        }
    }
    die "Wrong user login/password !".$res_auth->content."!.\n";
}
print "OK identified\n";

foreach my $file (@ARGV) {
    # Bad hack for description
    my $description = 'picture';
    if ($file =~ /mep/) {
        $description = 'MEP picture from europarl website';
    }
    elsif ($file =~ /mp/) {
        $description = 'MP picture from national assembly website';
    }

    my $src_file = abs_path($file);
    my $dst_file = basename $file;
    my $url = 'http://www.laquadrature.net/wiki/index.php?title=Special%3AUpload';
    my $res = $ua->post($url,
                        Content_Type => 'form-data',
                        Content =>  [
                                        wpUploadFile => [$src_file],
                                        wpDestFile => $dst_file,
                                        wpUploadDescription => $description,
                                        wpIgnoreWarning => 'true',
                                        wpUpload => 'Upload file',
                                    ]);
    if (not ($res->is_success or $res->is_redirect)) {
        die "Error in request !".$res->message."! code=!".$res->code."!\n";
    }
    elsif ($res->content ne '' and $res->content !~ /Successful upload/) {
        die "Error page: !" . $res->content . "!\n";
    }
    else {
        print "OK image $file inserted.\n";
    }
}
